@@ -43,30 +43,53 @@ def get_pending_test_infections(model):
 
 def get_diagnostic_test_detected_infections_student(model):
     return model.positive_tests[model.Testing.diagnostic_test_type]['student']
-
-
 def get_diagnostic_test_detected_infections_teacher(model):
     return model.positive_tests[model.Testing.diagnostic_test_type]['teacher']
-
-
 def get_diagnostic_test_detected_infections_family_member(model):
-    return model.positive_tests[model.Testing.diagnostic_test_type]\
-        ['family_member']
+    return model.positive_tests[model.Testing.diagnostic_test_type]['family_member']
+def get_diagnostic_test_detected_infections_resident(model):
+    return model.positive_tests[model.Testing.diagnostic_test_type]['resident']
+def get_diagnostic_test_detected_infections_employee(model):
+    return model.positive_tests[model.Testing.diagnostic_test_type]['employee']
+def get_diagnostic_test_detected_infections_unistudent(model):
+    return model.positive_tests[model.Testing.diagnostic_test_type]['unistudent']
+def get_diagnostic_test_detected_infections_lecturer(model):
+    return model.positive_tests[model.Testing.diagnostic_test_type]['lecturer']
 
+diagnostic_test_detected_infections_funcs = {
+    'student':get_diagnostic_test_detected_infections_student,
+    'teacher':get_diagnostic_test_detected_infections_teacher,
+    'family_member':get_diagnostic_test_detected_infections_family_member,
+    'resident':get_diagnostic_test_detected_infections_resident,
+    'employee':get_diagnostic_test_detected_infections_employee,
+    'unistudent':get_diagnostic_test_detected_infections_unistudent,
+    'lecturer':get_diagnostic_test_detected_infections_lecturer
+}
 
 def get_preventive_test_detected_infections_student(model):
-    return model.positive_tests[model.Testing.preventive_screening_test_type]\
-        ['student']
-
-
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['student']
 def get_preventive_test_detected_infections_teacher(model):
-    return model.positive_tests[model.Testing.preventive_screening_test_type]\
-        ['teacher']
-
-
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['teacher']
 def get_preventive_test_detected_infections_family_member(model):
-    return model.positive_tests[model.Testing.preventive_screening_test_type]\
-        ['family_member']
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['family_member']
+def get_preventive_test_detected_infections_resident(model):
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['resident']
+def get_preventive_test_detected_infections_employee(model):
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['employee']
+def get_preventive_test_detected_infections_unistudent(model):
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['unistudent']
+def get_preventive_test_detected_infections_lecturer(model):
+    return model.positive_tests[model.Testing.preventive_screening_test_type]['lecturer']
+
+preventive_test_detected_infections_funcs = {
+    'student':get_preventive_test_detected_infections_student,
+    'teacher':get_preventive_test_detected_infections_teacher,
+    'family_member':get_preventive_test_detected_infections_family_member,
+    'resident':get_preventive_test_detected_infections_resident,
+    'employee':get_preventive_test_detected_infections_employee,
+    'unistudent':get_preventive_test_detected_infections_unistudent,
+    'lecturer':get_preventive_test_detected_infections_lecturer
+}
 
 
 # parameter sanity check functions
@@ -185,18 +208,26 @@ class SEIRX(Model):
     the SEIRX dynamics of pandemic spread in a facility. Note:
     all times are set to correspond to days
 
-    G: networkx undirected graph, interaction graph between agents. Edges have
-    to have edge the edge attribute 'contact_type' specifying the closeness of
-    contacts, which can be ['very far', 'far', 'intermediate' and 'close'].
-    Nodes have to have the node attribute 'type' which specifies the agent type
-    of the given node (for example 'student' or 'teacher' in a school scenario).
-    In addition, nodes can have the attribute 'unit', which assigns them to a
-    unit in space (for example a 'class' in a school scenario).
+    G : networkx undirected graph
+        Interaction graph between agents. Edges have to have edge the edge 
+        attribute 'contact_type' specifying the closeness of contacts, which can
+        be ['very far', 'far', 'intermediate' and 'close']. Nodes have to have 
+        the node attribute 'type' which specifies the agent type of the given 
+        node (for example 'student' or 'teacher' in a school scenario).
+        In addition, nodes can have the attribute 'unit', which assigns them to 
+        a unit in space (for example a 'class' in a school scenario).
 
-    verbosity: integer in [0, 1, 2], controls text output to std out to track
-    simulation progress and transmission dynamics. Default = 0.
+    verbosity : integer in [0, 1, 2]
+        Controls text output to std out to track simulation progress and 
+        transmission dynamics. Default = 0.
 
-    testing, default = 'diagnostic'
+    base_transmission_risk : float
+        Base probability to transmit an infection durig a contact of type 
+        "close" if no other measures or biological parameters influence the
+        infection risk.
+
+    testing : str
+        Determines the testing strategy of the model. Default: 'diagnostic'.
         'diagnostic':   only diagnostic tests for symptomatic agents
         'background':   adds background screens of all agents after a positive
                         diagnostic test
@@ -207,8 +238,10 @@ class SEIRX(Model):
         'background+preventive': preventive screens AND background screens on
                         top of diagnostic testing.
 
-    infection_duration, default = 11 NOTE: includes the time an agent is exposed
-    but not yet infectious at the beginning of an infection
+    infection_duration : int or [float, float] 
+        Parameter determining the infection duration. Default: 11 
+        NOTE: includes the time an agent is exposed but not yet infectious at 
+        the beginning of an infection.
         positive integer:   mean or median of the infection duration in days
         list of two floats: mean and standard deviation of a distribution
                             specifying the infection duration in days. These
@@ -216,8 +249,9 @@ class SEIRX(Model):
                             distribution from which the infection duration will
                             be drawn for every agent individually
 
-    exposure_duration, default = 4. Sets the time from transmission to becoming
-    infectious
+    exposure_duration : int or [float, float] 
+        Parameter determining the exposure duration. Default: 4. 
+        Sets the time from transmission to becoming infectious
         positive integer:   mean or median of the exposure duration in days
         list of two floats: mean and standard deviation of a distribution
                             specifying the exposure duration in days. These
@@ -225,9 +259,10 @@ class SEIRX(Model):
                             distributoin from which the exposure duration will
                             be drawn for every agent individually.
 
-    time_until_symptoms, default = 6. Sets the time from transmission to
-    (potentially) developing symptoms. Symptom probability has to be set for
-    each agent group individually using the parameter 'symptom_probability'
+    time_until_symptoms : int or [float, float] 
+        Sets the time from transmission to (potentially) developing symptoms. 
+        Default: 6. Symptom probability has to be set for each agent group 
+        individually using the parameter 'symptom_probability'
         positive integer:   mean or median of the time until symptoms in days
         list of two floats: mean and standard deviation of a distribution
                             specifying the time until symptoms in days. These
@@ -235,82 +270,157 @@ class SEIRX(Model):
                             distribution from which the time until symptoms will
                             be drawn for every agent individually.
 
-    quarantine_duration, default = 14. Positive integer, sets the time a
-    positively tested agent is quarantined in days
+    quarantine_duration : int
+        Positive integer, sets the time a positively tested agent is quarantined
+        in days. Default: 14. 
 
-    infection_risk_contact_type_weights: dictionary of the form
-    {'very_far':float, 'far':float, 'intermediate':float, 'close':float}
-    that sets transmission risk multipliers for different contact types of
-    agents specified in the contact network G. Default: {'very_far': 0.1,
+    subclinical_modifier : float
+        Modifies the infectiousness of asymptomatic cases. Example: if 
+        subclinical_modifier = 0.5, the infectiousness of an asymptomatic case 
+        will be reduced to 50%. Default: 1.0.
+
+    infection_risk_contact_type_weights: dictionary 
+        Has to have the form
+        {
+        'very_far':    float, 
+        'far':         float, 
+        'intermediate':float, 
+        'close':       float
+        }
+    Sets transmission risk multipliers for different contact types of agents 
+    specified in the contact network G. Default: {'very_far': 0.1,
     'far': 0.5, 'intermediate': 1, 'close': 3}
 
-    subclinical_modifier: default = 1.0. Float, modifies the infectiousness of
-    asymptomatic cases. Example: if subclinical_modifier = 0.5, the
-    infectiousness of an asymptomatic case will be reduced to 50%.
 
-    K1_contact_types: list of strings from ['very_far', 'far', 'intermediate',
-    'close']. Definition of contact types for which agents are considered
-    "K1 contact persons" if they had contact to a positively tested person wtith
-    a specified contact intensity. Default = ['close'].
+    K1_contact_types : list of strings
+        Definition of contact types for which agents are considered "K1 contact 
+        persons" if they had contact to a positively tested person with a 
+        specified contact intensity. The list may only include strings from the
+        list ['very_far', 'far', 'intermediate', 'close'], i.e. the keys of the
+        infection_risk_contact_type_weights dictionary.  Default = ['close'].
 
-    diagnostic_test_type, default = 'one_day_PCR'. String, specifies the test
-    technology and test result turnover time used for diagnostic testing. For
-    example 'same_day_antigen' or 'two_day_PCR'. See module "Testing" for
-    different implemented testing techologies.
+    diagnostic_test_type : str 
+        Specifies the test technology and test result turnover time used for 
+        diagnostic testing. For example 'same_day_antigen' or 'two_day_PCR'. 
+        See module "Testing" for different implemented testing techologies.
+        Default: 'one_day_PCR'. 
 
-    preventive_screening_test_type:, default = 'one_day_PCR', String, specifies
-    the test technology and test result turnover time used for preventive
-    sreening. For example 'same_day_antigen' or 'two_day_PCR'. See module
-    "Testing" for different implemented testing techologies.
+    preventive_screening_test_type : str
+        Specifies the test technology and test result turnover time used for 
+        preventive sreening. For example 'same_day_antigen' or 'two_day_PCR'. 
+        See module "Testing" for different implemented testing techologies.
+        Default: 'one_day_PCR'.
 
-    follow_up_testing_interval, default = None. Positive integer, sets the time
-    a follow-up screen (background screen) is initiated after an initial screen
-    triggered by a positive test result. Only applies if the testing strategy is
-    'background' or preventive.
+    follow_up_testing_interval : int or None
+        Sets the time a follow-up screen (background screen) is initiated after 
+        an initial screen triggered by a positive test result. Only applies if 
+        the testing strategy is 'background' or 'preventive'. If "None" is 
+        chosen, no background screens are performed. Default: None. 
 
-    liberating_testing, default = False. Boolean, flag that specifies, whether
-    or not an agent is released from quarantine after returning a negative test
-    result.
+    liberating_testing : bool 
+        Flag that specifies, whether or not an agent is released from quarantine 
+        after returning a negative test result. Default: False.
 
-	index_case, default = 'employee' (nursing home scenario) or 'teacher'
-    (school scenario). Specifies how infections are introduced into the facility.
-        agent_type:     If an agent type (for example 'student' or 'teacher' in
-                        the school scenario) is specified, a single randomly
-                        chosen agent from this agent group will become the index
-                        case and no further index cases will be introduced into
-                        the scenario.
+	index_case: str
+        Specifies how infections are introduced into the facility. Can bei 
+        either a scenario-specific agent type (for example "student" or 
+        "teacher" in the school scenario) or "continuous". 
+        agent_type:     If an agent type is specified, a single randomly chosen 
+                        agent from this agent group will become the index case 
+                        and no further index cases will be introduced into the 
+                        scenario.
         'continuous':   In this case, agents have a continuous risk to become
                         index cases in every simulation step. The risk has to
                         be specified for every agent group individually, using
                         the 'index_probability' parameter. If only a single
                         agent group has a non-zero index probability, then only
                         agents from this group can become index cases.
+        Default = 'employee' (nursing home scenario), 'teacher' (school 
+        scenario), 'lecturer' (university scenario). 
 
 
-    agent_types: dictionary of the structure
+    agent_types: dictionary of dictionaries
+        Specifies the parameters 'screening_interval', 'index_probability', 
+        'mask' and 'vaccination_probability' individually for every agent group
+        in the model. The dictionary therefore needs to have an entry for every
+        agent type and every entry needs to have the following form:
         {
-        agent type:
+        agent_type:
             {
-            screening interval : integer, number of days between each preventive
-            screen in this agent group
-
-            index probability : float in the range [0, 1], sets the probability
-            to become an index case in each time step
-
+            screening_interval : int
+                Number of days between each preventive screen in this agent 
+                group.
+            index_probability : float 
+                Probability to become an index case in each time step. Needs to
+                be in in the range [0, 1]. 
             mask : bool
-                whether or not the agent type is wearing a mask
+                Whether or not the agent type is wearing a mask.
+            vaccination_probability : float
+                Probability of an agent in the agent group to be vaccinated.
+                Needs to be in the range [0, 1].
             }
         }
-
     The dictionary's keys are the names of the agent types which have to
-    correspond to the node attributes in the contact graph. The screening
-    interval sets the time-delay between preventive screens of this agent group,
-    the index probability sets the probability of a member of this agent group
-    becoming an index case in every time step
+    correspond to the node attributes in the contact graph. 
 
-    seed: positive integer, fixes the seed of the simulation to enable
-    repeatable simulation runs. If seed = None, the simulation will be
-    initialized at random.
+    age_transmission_risk_discount : dictionary
+        Specifies the linear relationship between agent age and transmission 
+        risk. Applies both to the risk of transmitting and the risk of receiving 
+        an infection. This is only used in scenarios in which agents have an age
+        attribute (school). The dictionary needs to have the following fields:
+            'slope' : float
+                The slope of the linear relationship.
+            'intercept' : float
+                The intercept of the linear relationship.
+        Default: {'slope':-0.02, 'intercept:1'}
+
+
+    age_symptom_modification : dictionary
+        Specifies the linear relationship between agent age and the probability
+        to develop symptoms. This is only used in scenarios in which agents have
+        an age attribute (school). The dictionary needs to have the following
+        fields:
+            'slope' : float
+                The slope of the linear relationship.
+            'intercept' : float
+                The intercept of the linear relationship.
+        Default: {'slope':-0.02545, 'intercept:0.854545'}
+
+    mask_filter_efficiency : dictionary
+        Specifies the effectiveness of masks to prevent the transmission and
+        reception of an infection. The dictionary needs to have the following
+        fields:
+            'inhale' : float
+                Reduction of the risk to become infected if the receiving agent
+                wears a mask.
+            'exhale' : float
+                Reduction of the risk to transmit an infection if the 
+                transmitting agent wears a mask.
+        Default: {'inhale':0, 'exhale':0}
+
+    transmission_risk_ventilation_modifier : float
+        Reduction of the transmission risk due to ventilation. Default: 0.
+
+    transmission_risk_vaccination_modifier : dictionary
+        Reduction of the transmission and reception risk due to vaccination.
+        The dictionary needs to have the following fields:
+            'reception' : float
+                Reduction of the probability to get infected if the infectee is
+                vaccinated.
+            'transmission' : float
+                Reduction of the probability to transmit the infection if the
+                infected agent is vaccinated.
+        Default: {'reception':1, 'transmission':0}
+
+    N_days_in_network : int
+        Number of distinct days the graph object contains. If the simulation
+        duration (in days) surpasses the number of days in the graph, the
+        simulation loops back to the first day etc. Default: 7.
+
+    seed : int
+        Fixes the seed of the simulation to enable repeatable simulation runs. 
+        If seed = None, the simulation will be initialized at random.
+        Default: None.
     '''
 
     def __init__(self, G,
@@ -320,7 +430,6 @@ class SEIRX(Model):
         exposure_duration = [5.0, 1.9],
         time_until_symptoms = [6.4, 0.8],
         infection_duration = [10.91, 3.95],
-        #vaccinated = False,
         quarantine_duration = 10,
         subclinical_modifier = 0.6,
         infection_risk_contact_type_weights = {
@@ -358,6 +467,7 @@ class SEIRX(Model):
         transmission_risk_vaccination_modifier = {
             'reception':1,
             'transmission':0},
+        N_days_in_network = 7,
         seed = None):
 
         # mesa models already implement fixed seeds through their own random
@@ -387,14 +497,14 @@ class SEIRX(Model):
         # internal step counter used to launch screening tests
         self.Nstep = 0
 
-        # since we may have weekday-specific contact networks, we need
+        # since we may have day-specific contact networks, we need
         # to keep track of the day of the week. Since the index case
         # per default is introduced at step 0 in index case mode, we
-        # need to offset the starting weekday by a random number of weekdays
+        # need to offset the starting day by a random number of days
         # to prevent artifacts from always starting on the same day of the week
-
-        self.weekday_offset = self.random.randint(1, 8)
-        self.weekday = self.Nstep + self.weekday_offset
+        self.N_days_in_network = N_days_in_network
+        self.day_offset = self.random.randint(1, 8)
+        self.day = self.Nstep + self.day_offset
 
         ## epidemiological parameters: can be either a single integer or the
         # mean and standard deviation of a distribution
@@ -481,6 +591,12 @@ class SEIRX(Model):
         if 'family_member' in agent_types:
             from scseirx.agent_family_member import family_member
             self.agent_classes['family_member'] = family_member
+        if 'lecturer' in agent_types:
+            from scseirx.agent_lecturer import lecturer
+            self.agent_classes['lecturer'] = lecturer
+        if 'unistudent' in agent_types:
+            from scseirx.agent_unistudent import unistudent
+            self.agent_classes['unistudent'] = unistudent
 
         ## set agent characteristics for all agent groups
         # list of agent characteristics
@@ -675,28 +791,28 @@ class SEIRX(Model):
 
         # data collectors to save population counts and agent states every
         # time step
-        self.datacollector = DataCollector(
-            model_reporters=
-            	{
-            	'N_diagnostic_tests':get_N_diagnostic_tests,
+
+        model_reporters = {
+                'N_diagnostic_tests':get_N_diagnostic_tests,
                 'N_preventive_screening_tests':get_N_preventive_screening_tests,
-                'diagnostic_test_detected_infections_student':\
-                        get_diagnostic_test_detected_infections_student,
-                'diagnostic_test_detected_infections_teacher':\
-                        get_diagnostic_test_detected_infections_teacher,
-                'diagnostic_test_detected_infections_family_member':\
-                        get_diagnostic_test_detected_infections_family_member,
-                'preventive_test_detected_infections_student':\
-                        get_preventive_test_detected_infections_student,
-                'preventive_test_detected_infections_teacher':\
-                        get_preventive_test_detected_infections_teacher,
-                'preventive_test_detected_infections_family_member':\
-                        get_preventive_test_detected_infections_family_member,
                 'undetected_infections':get_undetected_infections,
                 'predetected_infections':get_predetected_infections,
                 'pending_test_infections':get_pending_test_infections
-                },
+                }
 
+        for agent_type in self.agent_types:
+            model_reporters.update({
+                'diagnostic_test_detected_infections_{}'.format(agent_type):\
+                diagnostic_test_detected_infections_funcs[agent_type]
+                })
+            model_reporters.update({
+                'preventive_test_detected_infections_{}'.format(agent_type):\
+                preventive_test_detected_infections_funcs[agent_type]
+                })
+
+
+        self.datacollector = DataCollector(
+            model_reporters=model_reporters,
             agent_reporters=
             	{
             	'infection_state': get_infection_state,
@@ -706,13 +822,13 @@ class SEIRX(Model):
 
     ## transmission risk modifiers
     def get_transmission_risk_contact_type_modifier(self, source, target):
-        # construct the edge key as combination between agent IDs and weekday
+        # construct the edge key as combination between agent IDs and day
         n1 = source.ID
         n2 = target.ID
         tmp = [n1, n2]
         tmp.sort()
         n1, n2 = tmp
-        key = n1 + n2 + 'd{}'.format(self.weekday)
+        key = '{}{}d{}'.format(n1, n2, self.day)
         contact_weight = self.G.get_edge_data(n1, n2, key)['weight']
 
         # the link weight is a multiplicative modifier of the link strength.
@@ -967,16 +1083,17 @@ class SEIRX(Model):
         # find all agents that share edges with the agent
         # that are classified as K1 contact types in the testing
         # strategy
-        K1_contacts = [e[1] for e in self.G.edges(a.ID, data=True) if
-            e[2]['contact_type'] in self.Testing.K1_contact_types]
-        K1_contacts = [a for a in self.schedule.agents if a.ID in K1_contacts]
+        if a in self.G.nodes():
+            K1_contacts = [e[1] for e in self.G.edges(a.ID, data=True) if
+                e[2]['contact_type'] in self.Testing.K1_contact_types]
+            K1_contacts = [a for a in self.schedule.agents if a.ID in K1_contacts]
 
-        for K1_contact in K1_contacts:
-            if self.verbosity > 0:
-                print('quarantined {} {} (K1 contact of {} {})'
-                    .format(K1_contact.type, K1_contact.ID, a.type, a.ID))
-            K1_contact.quarantined = True
-            K1_contact.quarantine_start = self.Nstep
+            for K1_contact in K1_contacts:
+                if self.verbosity > 0:
+                    print('quarantined {} {} (K1 contact of {} {})'
+                        .format(K1_contact.type, K1_contact.ID, a.type, a.ID))
+                K1_contact.quarantined = True
+                K1_contact.quarantine_start = self.Nstep
 
     def test_symptomatic_agents(self):
         # find symptomatic agents that have not been tested yet and are not
@@ -1013,15 +1130,15 @@ class SEIRX(Model):
 
 
     def step(self):
-        self.weekday = (self.Nstep + self.weekday_offset) % 7 + 1
+        self.day = (self.Nstep + self.day_offset) % self.N_days_in_network + 1
         # if the connection graph is time-resloved, set the graph that is
         # used to determine connections in this step to the sub-graph corres-
         # ponding to the current day of the week
         if self.dynamic_connections:
-            self.G = self.weekday_connections[self.weekday]
+            self.G = self.day_connections[self.day]
 
         if self.verbosity > 0:
-            print('weekday {}'.format(self.weekday))
+            print('day {}'.format(self.day))
 
         if self.testing:
             for agent_type in self.agent_types:
@@ -1083,17 +1200,17 @@ class SEIRX(Model):
                         .format(interval, agent_type)
 
                     # (c.1) testing every 7 days = testing on Mondays
-                    if interval == 7 and self.weekday == 1:
+                    if interval == 7 and self.day % interval == 1:
                         self.screen_agents(agent_type,
                             self.Testing.preventive_screening_test_type,\
                              'preventive')
                     # (c.2) testing every 3 days = testing on Mo & Turs
-                    elif interval == 3 and self.weekday in [1, 4]:
+                    elif interval == 3 and self.day % interval in [1, 4]:
                             self.screen_agents(agent_type,
                             self.Testing.preventive_screening_test_type,\
                              'preventive')
                     # (c.3) testing every 2 days = testing on Mo, Wed & Fri
-                    elif interval == 2 and self.weekday in [1, 3, 5]:
+                    elif interval == 2 and self.day % interval in [1, 3, 5]:
                             self.screen_agents(agent_type,
                             self.Testing.preventive_screening_test_type,\
                              'preventive')
@@ -1103,7 +1220,7 @@ class SEIRX(Model):
                         pass
                     else:
                         if self.verbosity > 0:
-                            print('not initiating {} preventive screen (wrong weekday)'\
+                            print('not initiating {} preventive screen (wrong day)'\
                                     .format(agent_type))
             else:
                 # do nothing
